package geniusweb.profileconnection;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;

import javax.websocket.DeploymentException;
import javax.websocket.Session;
import javax.websocket.WebSocketContainer;

import org.junit.Test;

import geniusweb.profile.Profile;
import geniusweb.profileconnection.WebsocketProfileConnector;
import tudelft.utilities.logging.Reporter;

public class WebsocketProfileConnectorTest {
	private final Session session = mock(Session.class);
	private final Reporter reporter = mock(Reporter.class);
	private final WebSocketContainer wscontainer = mock(
			WebSocketContainer.class);
	private final static String testfilepath = "src/test/resources/testprofile.json";

	@Test(expected = IOException.class)
	public void testNoSuchURI()
			throws IOException, DeploymentException, URISyntaxException {
		when(wscontainer.connectToServer(any(Object.class), any(URI.class)))
				.thenThrow(new IOException("not available"));
		new WebsocketProfileConnector(new URI("no://where"), reporter,
				wscontainer);
	}

	@Test
	public void testOnOpen()
			throws IOException, DeploymentException, URISyntaxException {
		WebsocketProfileConnector connector = new WebsocketProfileConnector(
				new URI("ws://someprofile"), reporter, wscontainer);
		connector.onOpen(session);
		assertNull(connector.getProfile());
	}

	@Test
	public void testProcessMessage()
			throws IOException, DeploymentException, URISyntaxException {
		WebsocketProfileConnector connector = new WebsocketProfileConnector(
				new URI("ws://someprofile"), reporter, wscontainer);
		connector.onOpen(session);
		connector.processMessage(readTestProfile());

		Profile profile = connector.getProfile();
		assertNotNull(profile);
		assertEquals("testprofile", connector.getProfile().getName());
		assertEquals("test", connector.getProfile().getDomain().getName());
	}

	@Test
	public void testGetProfileWaiting()
			throws DeploymentException, IOException, URISyntaxException {
		/**
		 * Test that getProfile waits a little if necessary. We assume it waits
		 * at least 1 second for the profile to arrive
		 */
		WebsocketProfileConnector connector = new WebsocketProfileConnector(
				new URI("ws://someprofile"), reporter, wscontainer);
		connector.onOpen(session);

		new Thread(new Runnable() {

			@Override
			public void run() {
				try {
					Thread.sleep(1500);
					connector.processMessage(readTestProfile());
				} catch (InterruptedException | IOException e) {
					e.printStackTrace();
				}
			}
		}).start();
		Profile profile = connector.getProfile();
		assertNotNull(profile);
		assertEquals("testprofile", connector.getProfile().getName());
		assertEquals("test", connector.getProfile().getDomain().getName());
	}

	@Test
	public void testGetProfileTimeout()
			throws DeploymentException, IOException, URISyntaxException {
		/**
		 * Test that getProfile does time out if the profile does not come in
		 * quick enough.
		 */
		WebsocketProfileConnector connector = new WebsocketProfileConnector(
				new URI("ws://someprofile"), reporter, wscontainer);
		connector.onOpen(session);

		new Thread(new Runnable() {

			@Override
			public void run() {
				try {
					Thread.sleep(6000);
					connector.processMessage(readTestProfile());
				} catch (InterruptedException | IOException e) {
					e.printStackTrace();
				}
			}
		}).start();
		Profile profile = connector.getProfile();
		assertNull(profile);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testNullArg1()
			throws DeploymentException, IOException, URISyntaxException {
		new WebsocketProfileConnector(null, reporter, wscontainer);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testNullArg2()
			throws DeploymentException, IOException, URISyntaxException {
		new WebsocketProfileConnector(new URI("ws://someprofile"), null,
				wscontainer);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testNullArg3()
			throws DeploymentException, IOException, URISyntaxException {
		new WebsocketProfileConnector(new URI("ws://someprofile"), reporter,
				null);
	}

	private String readTestProfile() throws IOException {
		return new String(Files.readAllBytes(Paths.get(testfilepath)),
				StandardCharsets.UTF_8);

	}
}
